'use strict';

const regeneratorRuntime = require('regenerator-runtime');
const rollup = require('rollup');
const dts = require('rollup-plugin-dts');
const pathe = require('pathe');
const promises = require('fs/promises');
const alias = require('@rollup/plugin-alias');
const commonjs = require('@rollup/plugin-commonjs');
const json = require('@rollup/plugin-json');
const resolve = require('@rollup/plugin-node-resolve');
const replace = require('@rollup/plugin-replace');
const pluginutils = require('@rollup/pluginutils');
const core = require('@swc/core');
const fs = require('fs');
const merge = require('lodash.merge');
const jiti = require('jiti');
const Module = require('module');
const mri = require('mri');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e["default"] : e; }

const regeneratorRuntime__default = /*#__PURE__*/_interopDefaultLegacy(regeneratorRuntime);
const dts__default = /*#__PURE__*/_interopDefaultLegacy(dts);
const alias__default = /*#__PURE__*/_interopDefaultLegacy(alias);
const commonjs__default = /*#__PURE__*/_interopDefaultLegacy(commonjs);
const json__default = /*#__PURE__*/_interopDefaultLegacy(json);
const resolve__default = /*#__PURE__*/_interopDefaultLegacy(resolve);
const replace__default = /*#__PURE__*/_interopDefaultLegacy(replace);
const merge__default = /*#__PURE__*/_interopDefaultLegacy(merge);
const jiti__default = /*#__PURE__*/_interopDefaultLegacy(jiti);
const Module__default = /*#__PURE__*/_interopDefaultLegacy(Module);
const mri__default = /*#__PURE__*/_interopDefaultLegacy(mri);

var defineOutput = function(output) {
    return output;
};

function asyncGeneratorStep$9(gen, resolve, reject, _next, _throw, key, arg) {
    try {
        var info = gen[key](arg);
        var value = info.value;
    } catch (error) {
        reject(error);
        return;
    }
    if (info.done) {
        resolve(value);
    } else {
        Promise.resolve(value).then(_next, _throw);
    }
}
function _asyncToGenerator$9(fn) {
    return function() {
        var self = this, args = arguments;
        return new Promise(function(resolve, reject) {
            var gen = fn.apply(self, args);
            function _next(value) {
                asyncGeneratorStep$9(gen, resolve, reject, _next, _throw, "next", value);
            }
            function _throw(err) {
                asyncGeneratorStep$9(gen, resolve, reject, _next, _throw, "throw", err);
            }
            _next(undefined);
        });
    };
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any
var packageJson = {};
function loadPackage(rootDir, force) {
    return _loadPackage.apply(this, arguments);
}
function _loadPackage() {
    _loadPackage = _asyncToGenerator$9(regeneratorRuntime__default.mark(function _callee(rootDir, force) {
        var raw;
        return regeneratorRuntime__default.wrap(function _callee$(_ctx) {
            while(1)switch(_ctx.prev = _ctx.next){
                case 0:
                    if (!(packageJson.name && !force)) {
                        _ctx.next = 2;
                        break;
                    }
                    return _ctx.abrupt("return", Promise.resolve(packageJson));
                case 2:
                    _ctx.next = 4;
                    return promises.readFile(pathe.resolve(rootDir, "package.json"));
                case 4:
                    raw = _ctx.sent;
                    packageJson = JSON.parse(raw.toString());
                    return _ctx.abrupt("return", Promise.resolve(packageJson));
                case 7:
                case "end":
                    return _ctx.stop();
            }
        }, _callee);
    }));
    return _loadPackage.apply(this, arguments);
}

function asyncGeneratorStep$8(gen, resolve, reject, _next, _throw, key, arg) {
    try {
        var info = gen[key](arg);
        var value = info.value;
    } catch (error) {
        reject(error);
        return;
    }
    if (info.done) {
        resolve(value);
    } else {
        Promise.resolve(value).then(_next, _throw);
    }
}
function _asyncToGenerator$8(fn) {
    return function() {
        var self = this, args = arguments;
        return new Promise(function(resolve, reject) {
            var gen = fn.apply(self, args);
            function _next(value) {
                asyncGeneratorStep$8(gen, resolve, reject, _next, _throw, "next", value);
            }
            function _throw(err) {
                asyncGeneratorStep$8(gen, resolve, reject, _next, _throw, "throw", err);
            }
            _next(undefined);
        });
    };
}
function loadTsConfig(rootDir, filename) {
    return _loadTsConfig.apply(this, arguments);
}
function _loadTsConfig() {
    _loadTsConfig = _asyncToGenerator$8(regeneratorRuntime__default.mark(function _callee(rootDir, filename) {
        var resolved, readed, tsConfig, filepath, parent;
        return regeneratorRuntime__default.wrap(function _callee$(_ctx) {
            while(1)switch(_ctx.prev = _ctx.next){
                case 0:
                    resolved = pathe.resolve(rootDir, filename);
                    _ctx.next = 3;
                    return promises.readFile(resolved, {
                        encoding: "utf-8"
                    });
                case 3:
                    readed = _ctx.sent;
                    tsConfig = JSON.parse(readed);
                    tsConfig.compilerOptions = tsConfig.compilerOptions || {};
                    if (!(typeof tsConfig.extends === "string")) {
                        _ctx.next = 12;
                        break;
                    }
                    filepath = pathe.resolve(pathe.dirname(resolved), tsConfig.extends);
                    _ctx.next = 10;
                    return loadTsConfig(pathe.dirname(filepath), pathe.basename(filepath));
                case 10:
                    parent = _ctx.sent;
                    tsConfig.compilerOptions = Object.assign(parent.compilerOptions, tsConfig.compilerOptions);
                case 12:
                    return _ctx.abrupt("return", tsConfig);
                case 13:
                case "end":
                    return _ctx.stop();
            }
        }, _callee);
    }));
    return _loadTsConfig.apply(this, arguments);
}

function remove(path) {
    return promises.rm(path, {
        force: true,
        recursive: true
    });
}

var outputMapping = new Map();
var getRollupOutput = function(id) {
    return outputMapping.get(id);
};
function getRollupOutputs(options) {
    _registerDefaultOutputs(options.outDir);
    return options.outputs.map(function(out) {
        return _resolveOutput(options, out);
    });
}
function registerOutput(id, output, force) {
    if (!outputMapping.has(id) || force) {
        outputMapping.set(id, output);
    }
}
function _registerDefaultOutputs(outDir) {
    var cjsOutput = defineOutput({
        dir: outDir,
        entryFileNames: "[name].cjs",
        chunkFileNames: "chunks/[hash].cjs",
        format: "cjs",
        exports: "auto",
        preferConst: true,
        externalLiveBindings: false,
        freeze: false
    });
    var esmOutput = defineOutput({
        dir: outDir,
        entryFileNames: "[name].mjs",
        chunkFileNames: "chunks/[hash].mjs",
        format: "esm",
        exports: "auto",
        preferConst: true,
        externalLiveBindings: false,
        freeze: false
    });
    var dtsOutput = defineOutput({
        dir: outDir,
        format: "esm"
    });
    registerOutput("cjs", cjsOutput);
    registerOutput("esm", esmOutput);
    registerOutput("__dts__", dtsOutput);
}
function _resolveOutput(options, output) {
    if (typeof output !== "string") {
        var outDir = pathe.resolve(options.rootDir, output.dir || "");
        return Object.assign(output, {
            dir: outDir
        });
    }
    if (!outputMapping.has(output)) {
        console.error("".concat(output, " output type is not registered"));
        process.exit(1);
    }
    return outputMapping.get(output);
}

function asyncGeneratorStep$7(gen, resolve, reject, _next, _throw, key, arg) {
    try {
        var info = gen[key](arg);
        var value = info.value;
    } catch (error) {
        reject(error);
        return;
    }
    if (info.done) {
        resolve(value);
    } else {
        Promise.resolve(value).then(_next, _throw);
    }
}
function _asyncToGenerator$7(fn) {
    return function() {
        var self = this, args = arguments;
        return new Promise(function(resolve, reject) {
            var gen = fn.apply(self, args);
            function _next(value) {
                asyncGeneratorStep$7(gen, resolve, reject, _next, _throw, "next", value);
            }
            function _throw(err) {
                asyncGeneratorStep$7(gen, resolve, reject, _next, _throw, "throw", err);
            }
            _next(undefined);
        });
    };
}
function createBuilder(options, rollupOptions) {
    return _createBuilder.apply(this, arguments);
}
function _createBuilder() {
    _createBuilder = _asyncToGenerator$7(regeneratorRuntime__default.mark(function _callee(options, rollupOptions) {
        var parallelBuilds, allOutputOptions, buildResult, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, outputOptions, writter, ref, typesBuild, typesOutput, typesWritter;
        return regeneratorRuntime__default.wrap(function _callee$(_ctx) {
            while(1)switch(_ctx.prev = _ctx.next){
                case 0:
                    parallelBuilds = [];
                    allOutputOptions = rollupOptions.output;
                    _ctx.next = 4;
                    return rollup.rollup(rollupOptions);
                case 4:
                    buildResult = _ctx.sent;
                    console.info("Building for ".concat(options.target));
                    _iteratorNormalCompletion = true, _didIteratorError = false, _iteratorError = undefined;
                    _ctx.prev = 7;
                    for(_iterator = allOutputOptions[Symbol.iterator](); !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true){
                        outputOptions = _step.value;
                        writter = buildResult.write(outputOptions);
                        parallelBuilds.push(writter);
                    }
                    _ctx.next = 15;
                    break;
                case 11:
                    _ctx.prev = 11;
                    _ctx.t0 = _ctx["catch"](7);
                    _didIteratorError = true;
                    _iteratorError = _ctx.t0;
                case 15:
                    _ctx.prev = 15;
                    _ctx.prev = 16;
                    if (!_iteratorNormalCompletion && _iterator.return != null) {
                        _iterator.return();
                    }
                case 18:
                    _ctx.prev = 18;
                    if (!_didIteratorError) {
                        _ctx.next = 21;
                        break;
                    }
                    throw _iteratorError;
                case 21:
                    return _ctx.finish(18);
                case 22:
                    return _ctx.finish(15);
                case 23:
                    if (!options.declaration) {
                        _ctx.next = 32;
                        break;
                    }
                    (ref = rollupOptions.plugins) === null || ref === void 0 ? void 0 : ref.push(dts__default({}));
                    _ctx.next = 28;
                    return rollup.rollup(rollupOptions);
                case 28:
                    typesBuild = _ctx.sent;
                    typesOutput = getRollupOutput("__dts__");
                    typesWritter = typesBuild.write(typesOutput);
                    parallelBuilds.push(typesWritter);
                case 32:
                    return _ctx.abrupt("return", Promise.all(parallelBuilds));
                case 33:
                case "end":
                    return _ctx.stop();
            }
        }, _callee, null, [
            [
                7,
                11,
                15,
                23
            ],
            [
                16,
                ,
                18,
                22
            ]
        ]);
    }));
    return _createBuilder.apply(this, arguments);
}

function asyncGeneratorStep$6(gen, resolve, reject, _next, _throw, key, arg) {
    try {
        var info = gen[key](arg);
        var value = info.value;
    } catch (error) {
        reject(error);
        return;
    }
    if (info.done) {
        resolve(value);
    } else {
        Promise.resolve(value).then(_next, _throw);
    }
}
function _asyncToGenerator$6(fn) {
    return function() {
        var self = this, args = arguments;
        return new Promise(function(resolve, reject) {
            var gen = fn.apply(self, args);
            function _next(value) {
                asyncGeneratorStep$6(gen, resolve, reject, _next, _throw, "next", value);
            }
            function _throw(err) {
                asyncGeneratorStep$6(gen, resolve, reject, _next, _throw, "throw", err);
            }
            _next(undefined);
        });
    };
}
function createWatcher(options, rollupOptions) {
    if (Array.isArray(rollupOptions.output)) {
        rollupOptions.output = rollupOptions.output.map(function(out) {
            return _transformOutput(out);
        });
    }
    console.info("Watching for file changes...");
    var watcher = rollup.watch(rollupOptions);
    watcher.on("event", _createOnEventHandler(options));
    watcher.close();
    return watcher;
}
function _createOnEventHandler(options) {
    return function() {
        var _ref = _asyncToGenerator$6(regeneratorRuntime__default.mark(function _callee(event) {
            var ref, ref1;
            return regeneratorRuntime__default.wrap(function _callee$(_ctx) {
                while(1)switch(_ctx.prev = _ctx.next){
                    case 0:
                        if (!(event.code === "BUNDLE_START")) {
                            _ctx.next = 7;
                            break;
                        }
                        _ctx.next = 4;
                        return (ref = options.onBundleStart) === null || ref === void 0 ? void 0 : ref.call(options);
                    case 4:
                        console.info("Build started...");
                        _ctx.next = 16;
                        break;
                    case 7:
                        if (!(event.code === "BUNDLE_END")) {
                            _ctx.next = 15;
                            break;
                        }
                        event.result.close();
                        _ctx.next = 12;
                        return (ref1 = options.onBundleEnd) === null || ref1 === void 0 ? void 0 : ref1.call(options);
                    case 12:
                        console.info("Built in ".concat(event.duration, "ms."));
                        _ctx.next = 16;
                        break;
                    case 15:
                        if (event.code === "ERROR") {
                            console.error(event.error.message);
                        }
                    case 16:
                    case "end":
                        return _ctx.stop();
                }
            }, _callee);
        }));
        return function(event) {
            return _ref.apply(this, arguments);
        };
    }();
}
function _transformOutput(outputOptions) {
    var chunkFileNames = outputOptions.chunkFileNames;
    if (typeof chunkFileNames === "string") {
        outputOptions.chunkFileNames = chunkFileNames.replace("[hash]", "[name]");
    }
    return outputOptions;
}

function asyncGeneratorStep$5(gen, resolve, reject, _next, _throw, key, arg) {
    try {
        var info = gen[key](arg);
        var value = info.value;
    } catch (error) {
        reject(error);
        return;
    }
    if (info.done) {
        resolve(value);
    } else {
        Promise.resolve(value).then(_next, _throw);
    }
}
function _asyncToGenerator$5(fn) {
    return function() {
        var self = this, args = arguments;
        return new Promise(function(resolve, reject) {
            var gen = fn.apply(self, args);
            function _next(value) {
                asyncGeneratorStep$5(gen, resolve, reject, _next, _throw, "next", value);
            }
            function _throw(err) {
                asyncGeneratorStep$5(gen, resolve, reject, _next, _throw, "throw", err);
            }
            _next(undefined);
        });
    };
}
function swc(rootDir, tsConfig) {
    var options = arguments.length > 2 && arguments[2] !== void 0 ? arguments[2] : {};
    var filter = pluginutils.createFilter(/\.[jt]sx?$/, /node_modules/);
    var pathResolver = _createPathResolver(rootDir, options.extensions);
    return {
        name: "swc",
        resolveId: function resolveId(source, importer) {
            return _asyncToGenerator$5(regeneratorRuntime__default.mark(function _callee() {
                var resolvedPath;
                return regeneratorRuntime__default.wrap(function _callee$(_ctx) {
                    while(1)switch(_ctx.prev = _ctx.next){
                        case 0:
                            resolvedPath = pathResolver(source, importer);
                            if (!resolvedPath) {
                                resolvedPath = pathResolver(source, importer, true);
                            }
                            return _ctx.abrupt("return", resolvedPath);
                        case 3:
                        case "end":
                            return _ctx.stop();
                    }
                }, _callee);
            }))();
        },
        transform: function transform1(code, id) {
            return _asyncToGenerator$5(regeneratorRuntime__default.mark(function _callee() {
                var swcOptions, transformed;
                return regeneratorRuntime__default.wrap(function _callee$(_ctx) {
                    while(1)switch(_ctx.prev = _ctx.next){
                        case 0:
                            if (filter(id)) {
                                _ctx.next = 2;
                                break;
                            }
                            return _ctx.abrupt("return", null);
                        case 2:
                            swcOptions = _getSwcOptions(id, tsConfig, options);
                            _ctx.next = 5;
                            return core.transform(code, swcOptions);
                        case 5:
                            transformed = _ctx.sent;
                            return _ctx.abrupt("return", {
                                code: transformed.code,
                                map: transformed.map && JSON.parse(transformed.map)
                            });
                        case 7:
                        case "end":
                            return _ctx.stop();
                    }
                }, _callee);
            }))();
        },
        renderChunk: function renderChunk(code, chunk) {
            return _asyncToGenerator$5(regeneratorRuntime__default.mark(function _callee() {
                var transformed;
                return regeneratorRuntime__default.wrap(function _callee$(_ctx) {
                    while(1)switch(_ctx.prev = _ctx.next){
                        case 0:
                            if (options.minify) {
                                _ctx.next = 2;
                                break;
                            }
                            return _ctx.abrupt("return", null);
                        case 2:
                            _ctx.next = 4;
                            return core.transform(code, {
                                minify: true,
                                sourceMaps: true,
                                filename: chunk.fileName
                            });
                        case 4:
                            transformed = _ctx.sent;
                            return _ctx.abrupt("return", {
                                code: transformed.code,
                                map: transformed.map
                            });
                        case 6:
                        case "end":
                            return _ctx.stop();
                    }
                }, _callee);
            }))();
        }
    };
}
function _createPathResolver(rootDir) {
    var extensions = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : [];
    return function(source, importer, isIndex) {
        var resolvedPath;
        var basedir = importer ? pathe.dirname(importer) : rootDir;
        var filepath = pathe.resolve(basedir, source);
        var _iteratorNormalCompletion = true, _didIteratorError = false, _iteratorError = undefined;
        try {
            for(var _iterator = extensions[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true){
                var extension = _step.value;
                var _filepath = isIndex ? pathe.join(filepath, "index".concat(extension)) : pathe.join("".concat(extension).concat(extension));
                if (fs.existsSync(_filepath)) {
                    resolvedPath = _filepath;
                    break;
                }
            }
        } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
        } finally{
            try {
                if (!_iteratorNormalCompletion && _iterator.return != null) {
                    _iterator.return();
                }
            } finally{
                if (_didIteratorError) {
                    throw _iteratorError;
                }
            }
        }
        return resolvedPath;
    };
}
function _getSwcOptions(id, tsConfig, options) {
    var isTsFile = /\.tsx?$/.test(id);
    var compilerOptions = tsConfig === null || tsConfig === void 0 ? void 0 : tsConfig.compilerOptions;
    var defaultOptions = {
        filename: id,
        sourceMaps: true,
        jsc: {
            keepClassNames: compilerOptions === null || compilerOptions === void 0 ? void 0 : compilerOptions.experimentalDecorators,
            parser: {
                syntax: isTsFile ? "typescript" : "ecmascript",
                tsx: isTsFile,
                jsx: !isTsFile,
                decorators: compilerOptions === null || compilerOptions === void 0 ? void 0 : compilerOptions.experimentalDecorators
            },
            transform: {
                legacyDecorator: compilerOptions === null || compilerOptions === void 0 ? void 0 : compilerOptions.experimentalDecorators,
                decoratorMetadata: compilerOptions === null || compilerOptions === void 0 ? void 0 : compilerOptions.emitDecoratorMetadata,
                react: {
                    pragma: compilerOptions === null || compilerOptions === void 0 ? void 0 : compilerOptions.jsxFactory,
                    pragmaFrag: compilerOptions === null || compilerOptions === void 0 ? void 0 : compilerOptions.jsxFragmentFactory,
                    importSource: compilerOptions === null || compilerOptions === void 0 ? void 0 : compilerOptions.jsxImportSource
                }
            }
        }
    };
    return merge__default(defaultOptions, options);
}

function asyncGeneratorStep$4(gen, resolve, reject, _next, _throw, key, arg) {
    try {
        var info = gen[key](arg);
        var value = info.value;
    } catch (error) {
        reject(error);
        return;
    }
    if (info.done) {
        resolve(value);
    } else {
        Promise.resolve(value).then(_next, _throw);
    }
}
function _asyncToGenerator$4(fn) {
    return function() {
        var self = this, args = arguments;
        return new Promise(function(resolve, reject) {
            var gen = fn.apply(self, args);
            function _next(value) {
                asyncGeneratorStep$4(gen, resolve, reject, _next, _throw, "next", value);
            }
            function _throw(err) {
                asyncGeneratorStep$4(gen, resolve, reject, _next, _throw, "throw", err);
            }
            _next(undefined);
        });
    };
}
function _defineProperty$2(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _objectSpread$2(target) {
    for(var i = 1; i < arguments.length; i++){
        var source = arguments[i] != null ? arguments[i] : {};
        var ownKeys = Object.keys(source);
        if (typeof Object.getOwnPropertySymbols === "function") {
            ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function(sym) {
                return Object.getOwnPropertyDescriptor(source, sym).enumerable;
            }));
        }
        ownKeys.forEach(function(key) {
            _defineProperty$2(target, key, source[key]);
        });
    }
    return target;
}
function getRollupPlugins(options) {
    return _getRollupPlugins.apply(this, arguments);
}
function _getRollupPlugins() {
    _getRollupPlugins = _asyncToGenerator$4(regeneratorRuntime__default.mark(function _callee(options) {
        var ref, ref1, ref2, ref3, ref4, ref5, tsConfigJson;
        return regeneratorRuntime__default.wrap(function _callee$(_ctx) {
            while(1)switch(_ctx.prev = _ctx.next){
                case 0:
                    _ctx.next = 3;
                    return loadTsConfig(options.rootDir, options.tsConfig);
                case 3:
                    tsConfigJson = _ctx.sent;
                    return _ctx.abrupt("return", [
                        replace__default(_objectSpread$2({
                            true: true
                        }, (ref = options.rollupPlugins) === null || ref === void 0 ? void 0 : ref.replace)),
                        alias__default(_objectSpread$2({}, (ref1 = options.rollupPlugins) === null || ref1 === void 0 ? void 0 : ref1.alias)),
                        resolve__default(_objectSpread$2({
                            mainFields: [
                                "module",
                                "jsnext",
                                "main"
                            ],
                            browser: options.target === "browser",
                            extensions: options.extensions,
                            exportConditions: [
                                options.target === "node" ? "node" : "browser"
                            ],
                            preferBuiltins: options.target === "node"
                        }, (ref2 = options.rollupPlugins) === null || ref2 === void 0 ? void 0 : ref2.resolve)),
                        json__default(_objectSpread$2({
                            preferConst: true
                        }, (ref3 = options.rollupPlugins) === null || ref3 === void 0 ? void 0 : ref3.json)),
                        swc(options.rootDir, tsConfigJson, ((ref4 = options.rollupPlugins) === null || ref4 === void 0 ? void 0 : ref4.swc) || {}),
                        commonjs__default(_objectSpread$2({
                            include: [
                                /node_modules/
                            ],
                            extensions: options.extensions,
                            requireReturnsDefault: "namespace"
                        }, (ref5 = options.rollupPlugins) === null || ref5 === void 0 ? void 0 : ref5.commonjs))
                    ]);
                case 5:
                case "end":
                    return _ctx.stop();
            }
        }, _callee);
    }));
    return _getRollupPlugins.apply(this, arguments);
}

function asyncGeneratorStep$3(gen, resolve, reject, _next, _throw, key, arg) {
    try {
        var info = gen[key](arg);
        var value = info.value;
    } catch (error) {
        reject(error);
        return;
    }
    if (info.done) {
        resolve(value);
    } else {
        Promise.resolve(value).then(_next, _throw);
    }
}
function _asyncToGenerator$3(fn) {
    return function() {
        var self = this, args = arguments;
        return new Promise(function(resolve, reject) {
            var gen = fn.apply(self, args);
            function _next(value) {
                asyncGeneratorStep$3(gen, resolve, reject, _next, _throw, "next", value);
            }
            function _throw(err) {
                asyncGeneratorStep$3(gen, resolve, reject, _next, _throw, "throw", err);
            }
            _next(undefined);
        });
    };
}
function getRollupOptions(options, watch) {
    return _getRollupOptions.apply(this, arguments);
}
function _getRollupOptions() {
    _getRollupOptions = _asyncToGenerator$3(regeneratorRuntime__default.mark(function _callee(options, watch) {
        var rollupPlugins, defaultOptions;
        return regeneratorRuntime__default.wrap(function _callee$(_ctx) {
            while(1)switch(_ctx.prev = _ctx.next){
                case 0:
                    _ctx.next = 2;
                    return getRollupPlugins(options);
                case 2:
                    rollupPlugins = _ctx.sent;
                    defaultOptions = {
                        context: options.rootDir,
                        input: options.entries,
                        output: getRollupOutputs(options),
                        plugins: rollupPlugins,
                        watch: watch && _getWatchOptions(),
                        external: _createExternalFilter(options),
                        onwarn: _createWarnHandler()
                    };
                    return _ctx.abrupt("return", merge__default(defaultOptions, options.rollupOptions));
                case 5:
                case "end":
                    return _ctx.stop();
            }
        }, _callee);
    }));
    return _getRollupOptions.apply(this, arguments);
}
function _getWatchOptions() {
    return {
        chokidar: {
            ignoreInitial: true,
            ignorePermissionErrors: true,
            ignored: [
                "**/.git/**",
                "**/node_modules/**"
            ]
        }
    };
}
function _getPackageName(id) {
    var partials = id.split("/");
    return partials[0][0] === "@" ? "".concat(partials[0], "/").concat(partials[1]) : partials[0];
}
function _createExternalFilter(options) {
    return function(id) {
        var packageName = _getPackageName(id);
        var isExternal = options.externals.includes(packageName);
        if (options.inlineDependencies || id.startsWith(".") || id.startsWith("/")) {
            return false;
        }
        // if (!isExternal) {
        //     console.warn(`Inlining implicit external ${id}`);
        // }
        return isExternal;
    };
}
function _createWarnHandler() {
    var warningIgnoreList = [
        "CIRCULAR_DEPENDENCY",
        "EMPTY_BUNDLE",
        "THIS_IS_UNDEFINED"
    ];
    return function(warning, handler) {
        if (warning.code === "UNRESOLVED_IMPORT") {
            var id = warning.source;
            var importer = warning.importer;
            if (!importer || !/\?commonjs-external$/.test(importer)) {
                console.error("Rollup failed to resolve import \"".concat(id, "\" from \"").concat(importer, "\""));
                process.exit(-1);
            }
        }
        if (!warningIgnoreList.includes(warning.code)) {
            if (warning.code === "PLUGIN_WARNING") {
                console.warn("[plugin:".concat(warning.plugin, "] ").concat(warning.message));
            } else {
                handler(warning);
            }
        }
    };
}

function asyncGeneratorStep$2(gen, resolve, reject, _next, _throw, key, arg) {
    try {
        var info = gen[key](arg);
        var value = info.value;
    } catch (error) {
        reject(error);
        return;
    }
    if (info.done) {
        resolve(value);
    } else {
        Promise.resolve(value).then(_next, _throw);
    }
}
function _asyncToGenerator$2(fn) {
    return function() {
        var self = this, args = arguments;
        return new Promise(function(resolve, reject) {
            var gen = fn.apply(self, args);
            function _next(value) {
                asyncGeneratorStep$2(gen, resolve, reject, _next, _throw, "next", value);
            }
            function _throw(err) {
                asyncGeneratorStep$2(gen, resolve, reject, _next, _throw, "throw", err);
            }
            _next(undefined);
        });
    };
}
function build(options) {
    return _build.apply(this, arguments);
}
function _build() {
    _build = _asyncToGenerator$2(regeneratorRuntime__default.mark(function _callee(options) {
        var rollupOptions;
        return regeneratorRuntime__default.wrap(function _callee$(_ctx) {
            while(1)switch(_ctx.prev = _ctx.next){
                case 0:
                    _ctx.next = 2;
                    return getRollupOptions(options);
                case 2:
                    rollupOptions = _ctx.sent;
                    _ctx.next = 5;
                    return remove(options.outDir);
                case 5:
                    return _ctx.abrupt("return", createBuilder(options, rollupOptions));
                case 6:
                case "end":
                    return _ctx.stop();
            }
        }, _callee);
    }));
    return _build.apply(this, arguments);
}

function asyncGeneratorStep$1(gen, resolve, reject, _next, _throw, key, arg) {
    try {
        var info = gen[key](arg);
        var value = info.value;
    } catch (error) {
        reject(error);
        return;
    }
    if (info.done) {
        resolve(value);
    } else {
        Promise.resolve(value).then(_next, _throw);
    }
}
function _asyncToGenerator$1(fn) {
    return function() {
        var self = this, args = arguments;
        return new Promise(function(resolve, reject) {
            var gen = fn.apply(self, args);
            function _next(value) {
                asyncGeneratorStep$1(gen, resolve, reject, _next, _throw, "next", value);
            }
            function _throw(err) {
                asyncGeneratorStep$1(gen, resolve, reject, _next, _throw, "throw", err);
            }
            _next(undefined);
        });
    };
}
function watch(options) {
    return _watch.apply(this, arguments);
}
function _watch() {
    _watch = _asyncToGenerator$1(regeneratorRuntime__default.mark(function _callee(options) {
        var rollupOptions;
        return regeneratorRuntime__default.wrap(function _callee$(_ctx) {
            while(1)switch(_ctx.prev = _ctx.next){
                case 0:
                    _ctx.next = 2;
                    return getRollupOptions(options, true);
                case 2:
                    rollupOptions = _ctx.sent;
                    _ctx.next = 5;
                    return remove(options.outDir);
                case 5:
                    return _ctx.abrupt("return", createWatcher(options, rollupOptions));
                case 6:
                case "end":
                    return _ctx.stop();
            }
        }, _callee);
    }));
    return _watch.apply(this, arguments);
}

var configNames = new Set([
    ".bauenrc.js",
    ".bauenrc.cjs",
    ".bauenrc.mjs",
    ".bauenrc.ts",
    "bauen.config.js",
    "bauen.config.cjs",
    "bauen.config.mjs",
    "bauen.config.ts"
]);
function searchConfig(rootDir) {
    var result = {
        name: null,
        path: null
    };
    var _iteratorNormalCompletion = true, _didIteratorError = false, _iteratorError = undefined;
    try {
        for(var _iterator = configNames[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true){
            var name = _step.value;
            var path = pathe.resolve(rootDir, name);
            if (fs.existsSync(path)) {
                Object.assign(result, {
                    name: name,
                    path: path
                });
                break;
            }
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally{
        try {
            if (!_iteratorNormalCompletion && _iterator.return != null) {
                _iterator.return();
            }
        } finally{
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }
    return result;
}

function loadConfig(rootDir) {
    var _require = jiti__default(rootDir, {
        interopDefault: true
    });
    var foundConfig = searchConfig(rootDir);
    try {
        return _require(foundConfig.path);
    } catch (err) {
        console.error("Config file not found in ".concat(rootDir), err);
        process.exit(1);
    }
}

function _arrayLikeToArray$1(arr, len) {
    if (len == null || len > arr.length) len = arr.length;
    for(var i = 0, arr2 = new Array(len); i < len; i++)arr2[i] = arr[i];
    return arr2;
}
function _arrayWithoutHoles$1(arr) {
    if (Array.isArray(arr)) return _arrayLikeToArray$1(arr);
}
function _defineProperty$1(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _iterableToArray$1(iter) {
    if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter);
}
function _nonIterableSpread$1() {
    throw new TypeError("Invalid attempt to spread non-iterable instance.\\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}
function _objectSpread$1(target) {
    for(var i = 1; i < arguments.length; i++){
        var source = arguments[i] != null ? arguments[i] : {};
        var ownKeys = Object.keys(source);
        if (typeof Object.getOwnPropertySymbols === "function") {
            ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function(sym) {
                return Object.getOwnPropertyDescriptor(source, sym).enumerable;
            }));
        }
        ownKeys.forEach(function(key) {
            _defineProperty$1(target, key, source[key]);
        });
    }
    return target;
}
function _objectWithoutProperties(source, excluded) {
    if (source == null) return {};
    var target = _objectWithoutPropertiesLoose(source, excluded);
    var key, i;
    if (Object.getOwnPropertySymbols) {
        var sourceSymbolKeys = Object.getOwnPropertySymbols(source);
        for(i = 0; i < sourceSymbolKeys.length; i++){
            key = sourceSymbolKeys[i];
            if (excluded.indexOf(key) >= 0) continue;
            if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
            target[key] = source[key];
        }
    }
    return target;
}
function _objectWithoutPropertiesLoose(source, excluded) {
    if (source == null) return {};
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;
    for(i = 0; i < sourceKeys.length; i++){
        key = sourceKeys[i];
        if (excluded.indexOf(key) >= 0) continue;
        target[key] = source[key];
    }
    return target;
}
function _toConsumableArray$1(arr) {
    return _arrayWithoutHoles$1(arr) || _iterableToArray$1(arr) || _unsupportedIterableToArray$1(arr) || _nonIterableSpread$1();
}
function _unsupportedIterableToArray$1(o, minLen) {
    if (!o) return;
    if (typeof o === "string") return _arrayLikeToArray$1(o, minLen);
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if (n === "Object" && o.constructor) n = o.constructor.name;
    if (n === "Map" || n === "Set") return Array.from(n);
    if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray$1(o, minLen);
}
function getCliOptions(argv) {
    var args = mri__default(_toConsumableArray$1(argv).splice(2));
    var _ = args._, restArgs = _objectWithoutProperties(args, [
        "_"
    ]);
    return _objectSpread$1({}, _.length > 0 && {
        command: _[0]
    }, restArgs);
}

function getDefaultOptions() {
    return {
        command: "build",
        rootDir: process.cwd(),
        outDir: "dist",
        entries: [],
        outputs: [],
        target: "node",
        extensions: [
            ".ts",
            ".tsx",
            ".mjs",
            ".cjs",
            ".js",
            ".jsx"
        ],
        externals: [],
        tsConfig: "tsconfig.json",
        declaration: true,
        inlineDependencies: false
    };
}

function _arrayLikeToArray(arr, len) {
    if (len == null || len > arr.length) len = arr.length;
    for(var i = 0, arr2 = new Array(len); i < len; i++)arr2[i] = arr[i];
    return arr2;
}
function _arrayWithHoles(arr) {
    if (Array.isArray(arr)) return arr;
}
function _arrayWithoutHoles(arr) {
    if (Array.isArray(arr)) return _arrayLikeToArray(arr);
}
function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
    try {
        var info = gen[key](arg);
        var value = info.value;
    } catch (error) {
        reject(error);
        return;
    }
    if (info.done) {
        resolve(value);
    } else {
        Promise.resolve(value).then(_next, _throw);
    }
}
function _asyncToGenerator(fn) {
    return function() {
        var self = this, args = arguments;
        return new Promise(function(resolve, reject) {
            var gen = fn.apply(self, args);
            function _next(value) {
                asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
            }
            function _throw(err) {
                asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
            }
            _next(undefined);
        });
    };
}
function _defineProperty(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _iterableToArray(iter) {
    if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter);
}
function _iterableToArrayLimit(arr, i) {
    var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"];
    if (_i == null) return;
    var _arr = [];
    var _n = true;
    var _d = false;
    var _s, _e;
    try {
        for(_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true){
            _arr.push(_s.value);
            if (i && _arr.length === i) break;
        }
    } catch (err) {
        _d = true;
        _e = err;
    } finally{
        try {
            if (!_n && _i["return"] != null) _i["return"]();
        } finally{
            if (_d) throw _e;
        }
    }
    return _arr;
}
function _nonIterableRest() {
    throw new TypeError("Invalid attempt to destructure non-iterable instance.\\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}
function _nonIterableSpread() {
    throw new TypeError("Invalid attempt to spread non-iterable instance.\\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}
function _objectSpread(target) {
    for(var i = 1; i < arguments.length; i++){
        var source = arguments[i] != null ? arguments[i] : {};
        var ownKeys = Object.keys(source);
        if (typeof Object.getOwnPropertySymbols === "function") {
            ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function(sym) {
                return Object.getOwnPropertyDescriptor(source, sym).enumerable;
            }));
        }
        ownKeys.forEach(function(key) {
            _defineProperty(target, key, source[key]);
        });
    }
    return target;
}
function _slicedToArray(arr, i) {
    return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();
}
function _toConsumableArray(arr) {
    return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread();
}
function _unsupportedIterableToArray(o, minLen) {
    if (!o) return;
    if (typeof o === "string") return _arrayLikeToArray(o, minLen);
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if (n === "Object" && o.constructor) n = o.constructor.name;
    if (n === "Map" || n === "Set") return Array.from(n);
    if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}
function getOptions(argv, userConfig) {
    return _getOptions.apply(this, arguments);
}
function _getOptions() {
    _getOptions = _asyncToGenerator(regeneratorRuntime__default.mark(function _callee(argv, userConfig) {
        var ref, defaultOptions, inputOptions, rootDir, tsConfig, ref1, packageJson;
        return regeneratorRuntime__default.wrap(function _callee$(_ctx) {
            while(1)switch(_ctx.prev = _ctx.next){
                case 0:
                    defaultOptions = getDefaultOptions();
                    inputOptions = _objectSpread({}, getCliOptions(argv), userConfig);
                    rootDir = pathe.resolve(inputOptions.rootDir || defaultOptions.rootDir);
                    tsConfig = pathe.resolve(rootDir, inputOptions.tsConfig || defaultOptions.tsConfig);
                    _ctx.t0 = _slicedToArray;
                    _ctx.next = 8;
                    return Promise.all([
                        loadPackage(rootDir),
                        loadTsConfig(rootDir, tsConfig)
                    ]);
                case 8:
                    _ctx.t1 = _ctx.sent;
                    ref1 = (0, _ctx.t0)(_ctx.t1, 1);
                    packageJson = ref1[0];
                    return _ctx.abrupt("return", merge__default(defaultOptions, inputOptions, {
                        rootDir: rootDir,
                        outDir: pathe.resolve(rootDir, inputOptions.outDir || defaultOptions.outDir),
                        entries: (ref = inputOptions.entries) === null || ref === void 0 ? void 0 : ref.map(function(entry) {
                            return pathe.resolve(rootDir, entry);
                        }),
                        externals: _getExternals(userConfig, packageJson)
                    }));
                case 12:
                case "end":
                    return _ctx.stop();
            }
        }, _callee);
    }));
    return _getOptions.apply(this, arguments);
}
function _getExternals(userConfig, packageJson) {
    if (userConfig.target === "browser") {
        return [];
    }
    return _toConsumableArray(Module__default.builtinModules).concat(_toConsumableArray(Object.keys(packageJson.dependencies || [])), _toConsumableArray(Object.keys(packageJson.peerDependencies || [])), _toConsumableArray(userConfig.externals || []));
}

exports.build = build;
exports.createBuilder = createBuilder;
exports.createWatcher = createWatcher;
exports.defineOutput = defineOutput;
exports.getCliOptions = getCliOptions;
exports.getDefaultOptions = getDefaultOptions;
exports.getOptions = getOptions;
exports.getRollupOptions = getRollupOptions;
exports.getRollupOutput = getRollupOutput;
exports.getRollupOutputs = getRollupOutputs;
exports.getRollupPlugins = getRollupPlugins;
exports.loadConfig = loadConfig;
exports.loadPackage = loadPackage;
exports.loadTsConfig = loadTsConfig;
exports.registerOutput = registerOutput;
exports.remove = remove;
exports.searchConfig = searchConfig;
exports.swc = swc;
exports.watch = watch;
